﻿using System;
using System.Windows.Threading;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace C1ToolbarSamples
{
    /// <summary>
    /// Derives from <see cref="C1ToolbarDropDown"/> but drops down when the mouse 
    /// hovers over the button (instead of when the user clicks on it).
    /// </summary>
    public class HoverDropDown : C1.Silverlight.Toolbar.C1ToolbarDropDown
    {
        Popup _helper;
        Rectangle _rect;
        DispatcherTimer _timer;

        public HoverDropDown()
        {
            // create timer to hide the dropdown when the mouse moves away
            // from the toolbar item itself and from the dropdown
            _timer = new DispatcherTimer();
            _timer.Interval = TimeSpan.FromSeconds(0.5);
            _timer.Tick += _timer_Tick;

            // create transparent element to detect mouse movement over the 
            // toolbar item
            _rect = new Rectangle();
            _rect.Fill = new SolidColorBrush(Colors.Transparent);
            _helper = new Popup();
            _helper.Child = _rect;

            // start/stop the timer when mouse leaves/enters the toolbar item
            _rect.MouseLeave += (s, e) => _timer.Start();
            _rect.MouseEnter += (s, e) => _timer.Stop();
        }

        // connect our timer to the dropdown element (popup child)
        public override void OnApplyTemplate()
        {
            // allow default handling
            base.OnApplyTemplate();

            // get the dropdown element from the template
            var popup = GetTemplateChild("DropDownPopup") as Popup;
            var popupChild = (FrameworkElement)popup.Child;

            // start/stop the timer when mouse leaves/enters the dropdown
            popupChild.MouseLeave += (s, e) => _timer.Start();
            popupChild.MouseEnter += (s, e) => _timer.Stop();
        }

        // drop down when mouse enters the toolbar item
        // control, setup to detect when the mouse leaves the dropdown element
        protected override void OnMouseEnter(MouseEventArgs e)
        {
            base.OnMouseEnter(e);
            if (!IsDropDownOpen)
            {
                // drop down now
                IsDropDownOpen = true;
                
                // update mouse detector position
                Point pt = TransformToVisual(null).Transform(new Point());
                _rect.Width = ActualWidth;
                _rect.Height = ActualHeight;
                _helper.HorizontalOffset = pt.X;
                _helper.VerticalOffset = pt.Y;
                _helper.IsOpen = true;
            }
        }

        // close popup 0.5 seconds after the mouse leaves the popup area
        void _timer_Tick(object sender, EventArgs e)
        {
            IsDropDownOpen = _helper.IsOpen = false;
        }
    }
}
